#include <tt8lib.h>
#include <stdio.h>
#include <stdlib.h>
#include "izm.h"
#include "latchusb.h"

#define _LO 0
#define _HI 1

/****************************************************************************/
/* ZMODEM Primitives                                                        */
/****************************************************************************/

/*
 *  Send a character to modem.  Small is beautiful, or is it?
 */
void sendline(char c)
{
      WrUsbDataByte(c);

      //first check for USB LINK STATE, pin8 of USB module
      //non-zero SLEEP state indicates USB bus is in standby, therefore bail
      //and return some value to ZMODEM that will close things down
//    if(RdUsbSleepPin() == _HI)
//       TRAP LOSS OF "CD" RET -3

//    PSUEDO BUFFER DUMP CODE
//    while(buffer has stuff){
//      if(RdUsbTxePin() == _LO){
//        WrUsbDataByte(bufval);
//       }
//      else{
//        we'll need a wait w/timeout
//       }
//    }
}

void mputs(char *s)
{
      while (*s)
        sendline(*s++);
}

/*
 * Purge the device's input buffer of all characters
 */
void purgeline(void)
{
      //if reciever has some data, purge it
      while(RdUsbRxfPin() == _LO){
        RdUsbDataByte();
        }
}

/*
 * This version of readline is reasoably well suited for
 * reading many characters.
 *  (except, currently, for the Regulus version!)
 *
 * timeout is passed in tenths of seconds
 */
int readline(int timeout)
{
      ulong  msTimeout, Timeout;
      short  d;

      //first check for USB LINK STATE, pin8 of USB module
      //non-zero SLEEP state indicates USB bus is in standby, therefore bail
      //and return some value to ZMODEM that will close things down
//    if(RdUsbSleepPin() == _HI)
//       TRAP LOSS OF "CD" RET -3

      msTimeout = (ulong) (timeout*100L);

      Timeout = MilliSecs() + msTimeout;

      while(RdUsbRxfPin() == _HI){
        if(MilliSecs() > Timeout){
          return TIMEOUT;
          }
        }

      d = (short) RdUsbDataByte();

      return (d & 0377);
}

int checkline(void)
{

      if(RdUsbRxfPin() == _HI)
        return(FALSE);
      else
        return(TRUE);
}

/****************************************************************************/
/* USB Status and Data Functions to be used by ZMODEM Primitives above      */
/****************************************************************************/

/*
 *  Read a character from the USB controller
 */
unsigned char RdUsbDataByte(void)
{
#asm
USBMEMLOC   equ     $401000             ;memory mapped USB Module, CS10
            move.w  USBMEMLOC,d0        ;get contents of latch
            andi.w  #$00FF,d0           ;mask it off
                                        ;return from C function call
#endasm
}

/*
 *  Send a character to the USB controller
 */
void WrUsbDataByte(unsigned char arg_val)
{
;
#asm
            move.b  %%arg_val,USBMEMLOC ;load contents into device
                                        ;return from C function call
#endasm
}

/*
 *  Read the state of the SLEEP line, pin 8 on the DLP USB Module
 *  SLEEP HI indicates the USB link is in suspend mode, give up, like CD in RS232 speeak
 *  SLEEP LO indicates the USB link is actice, all systems go
 */
unsigned char RdUsbSleepPin(void)
{
    return( (RdRegister1() & 0x0040) >> 6 );
}

/*
 *  Read the state of the RXF line, pin 13 on the DLP USB Module
 *  RXF HI indicates Rx FIFO is empty
 *  RXF LO indicates Rx FIFO has at least one char
 *  FIFO depth is 128 BYTES
 */
unsigned char RdUsbRxfPin(void)
{
    return( (RdRegister1() & 0x0020) >> 5 );
}

/*
 *  Read the state of the TXE line, pin 14 on the DLP USB Module
 *  TXE HI indicates Tx FIFO is full, don't write any more data
 *  TXE LO indicates device can be written to
 *  FIFO depth is 384 BYTES
 */
unsigned char RdUsbTxePin(void)
{
    return( (RdRegister1() & 0x0010) >> 4 );
}

