/*% cc -O -DLDP -DUNIX -DDIAGS %  -o isz; size isz;
 ****************************************************************************
 *
 * isz.c By Chuck Forsberg,  Omen Technology INC
 *  Copyright 2002 Omen Technology INC All Rights Reserved
 *  Entry: see sample main() in code below
 *
 *  External functions called:
 *
 * sendline(c)  sends a character to host
 * mputs(s) sends null terminated string to host
 *
 * purgeline()  empty receive queue
 * int readline(tenths) read char from host, timeout in seconds/10
        return received byte 0 to 0377
        negative return values indicate error:
                return -1 on recoverable error
                return -2 on timeout
        trap fatal errors (loss of carrier, etc.), close file
 * checkline() return non 0 if readline has 1 or more bytes ready
 *
 * zfilbuf() fill buffer with blklen bytes, set Eofseen if EOT, return count
 * zseek(whence) clear EOF, errors, seek
 * zclose(status) close the input dataset associated with zfilbuf() et al.
 *      N.B: zclose might be called after the file is closed.
 *
 * When trapping on errors, close file, call canit() unless line dropped
 *
 ****************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include "izm.h"
#include "latchusb.h"

extern int errno;
// I ADDED THIS
extern char secbuf[1025];
//
FILE *in;
int Verbose=0;
int Zmodem=0;           /* ZMODEM protocol requested by receiver */
unsigned Txwindow=4096; /* Control the size of the transmitted window */
unsigned Txwspac;       /* Spacing between zcrcq requests */
unsigned Txwcnt;        /* Counter used to space ack requests */
long vpos = 0;          /* Number of bytes read from file */
long Lrxpos;            /* Receiver's last reported offset */
int Lastrx;
char Lzconv;          /* Local ZMODEM file conversion request */
int blklen=1024;        /* length of transmitted records */
int Eofseen;            /* EOF seen on input set by zfilbuf */
unsigned Rxbuflen = 16384;      /* Receiver's max buffer length */
int Tframlen = 0;       /* Override for tx frame length */
int Rxflags = 0;
long Lastsync;          /* Last offset to which we got a ZRPOS */
int Beenhereb4;         /* How many times we've been ZRPOS'd same place */
int Zctlesc;          /* Encode control characters */
int Zrwindow = 1400;    /* RX window size (controls garbage count) */
int Rxtimeout = 600;

//#include "icrctab.c"

//#include "tt8_zmod.c"

//#include "izm.c"

//it ain't UNIX and this code ain't pretty but we might need this
//but who really knows?!
int errcnt=0;           /* number of files unreadable */


int SZ(char *fname, char *fsize)
{

        startz();               /* Initialize receiver */

        /*
         *  To ask receiver for crash rocovery, set
         *              Lzconv = ZCRESUM;
         *
         * call wcs() once for each file.
         *  Initialize zfilbuf before calling wcs()
         *  The string is the file name, null terminated
         *    file length is not sent
         */
        if(wcs(fname, fsize)){
          return(TRUE);
          }

        saybibi();              /* Orderly conclusion of ZMODEM session */

        return(FALSE);
        
}

startz()
{
#ifdef BOTH
        Rxtimeout = 600;
#endif
        Zmodem = 0;
#ifdef LDP
        Zldpactive = 0;
        Zcrcpreset = 0xFFFFFFFFL;       /* CRC preset from filename header */
#endif
        mputs("rz\r");
        stohdr(0L);
        zshhdr(ZRQINIT, Txhdr);
}

wcs(char *name, char *size)
{

        if ((in=fopen(name, "r"))==NULL) {
                ++errcnt;
                return ERROR;
        }

        Eofseen = 0; vpos = 0;

        switch (wctxpn(name, size)) {
        case ZSKIP:
        case ZFERR:
                return OK;
        case OK:
                break;
        default:
                return ERROR;
        }
        return 0;
}

/*
 * generate and transmit pathname block consisting of
 *  pathname (null terminated),
 */
wctxpn(char *name, char *size)
{

        long n, s;
        int length;

        if ( !Zmodem)
                if (getnak())
                        return ERROR;
        Zmodem = 1;

        sprintf(secbuf,"%s%c%s%c", name, 0, size, 0);
        n=strlen(name);
        s=strlen(size);
        length = n + s + 2;

        return zsendfile(secbuf, length);

}

getnak()
{
        register firstch;

        Lastrx = 0;
        for (;;) {
                switch (firstch = readline(800)) {
                case ZPAD:
                        if (getzrxinit())
                                return ERROR;
                        return FALSE;
                case TIMEOUT:
                        return TRUE;
                case CAN:
                        if ((firstch = readline(20)) == CAN && Lastrx == CAN)
                                return TRUE;
                default:
                        break;
                }
                Lastrx = firstch;
        }
}


zclose(status)
{
        if (in) {
          fclose(in);
          }
        in = 0;
}

int
zseek(n)
long n;
{
        clearerr(in);   /* Clear EOF */
        return (fseek(in, n, 0));
}

/* Fill buffer with blklen chars */
zfilbuf()
{
        int n;

        n = fread(secbuf, 1, blklen, in);
        if (n < blklen)
                Eofseen = 1;
        return n;
}

/*
 * Get the receiver's init parameters
 */
getzrxinit()
{
        register n;

        for (n=10; --n>=0; ) {
                
                switch (zgethdr()) {
                case ZCHALLENGE:        /* Echo receiver's challenge numbr */
                        stohdr(Rxpos);
                        zshhdr(ZACK, Txhdr);
                        continue;
                case ZRINIT:
#ifdef LDP
                        /* LDP protection active */
                        Zldpactive = Rxhdr[ZF1] & CANLDP;
#ifdef DIAGS
                        vfile("Zldpactive=%d", Zldpactive);
#endif
#endif
                        Rxflags = 0377 & Rxhdr[ZF0];
#ifndef XEDCRC32
                        Txfcs32 = Rxflags & CANFC32;
#endif
                        Zctlesc |= Rxflags & TESCCTL;
                        Rxbuflen = (0377 & Rxhdr[ZP0])+((0377 & Rxhdr[ZP1])<<8);
                        if ( !(Rxflags & CANFDX))
                                Txwindow = 0;
#ifdef DIAGS
                        vfile("Rxbuflen=%d Tframlen=%d", Rxbuflen, Tframlen);
#endif
                        /* Override to force shorter frame length */
                        if (Rxbuflen && (Rxbuflen>Tframlen) && (Tframlen>=32))
                                Rxbuflen = Tframlen;
                        if ( !Rxbuflen && (Tframlen>=32) && (Tframlen<=1024))
                                Rxbuflen = Tframlen;
#ifdef DIAGS
                        vfile("Rxbuflen=%d", Rxbuflen);
#endif

                        /* Set initial subpacket length */
                        if (Rxbuflen && blklen>Rxbuflen)
                                blklen = Rxbuflen;
#ifdef DIAGS
                        vfile("Rxbuflen=%d blklen=%d", Rxbuflen, blklen);
                        vfile("Txwindow = %u Txwspac = %d", Txwindow, Txwspac);
#endif

                        return OK;
                case ZCAN:
                case TIMEOUT:
                        return ERROR;
                default:
                        zshhdr(ZNAK, Txhdr);
                        continue;
                }
        }
        return ERROR;
}


/* Send file name and related info */
zsendfile(buf, blen)
char *buf;
{
        register c;
#ifdef LDP
        long crc;
        char *p;
#endif

        for (;;) {
                stohdr(0L);
                Txhdr[ZF0] = Lzconv;    /* file conversion request */
#ifdef LDP
                Zcrcpreset = crc = 0xFFFFFFFFL;
                bytcnt = 0;
                if (Zldpactive)
                        Txhdr[ZF3] |= ZSLDP;
#endif
                zsbhdr(ZFILE, Txhdr);
                zsdata(buf, blen, ZCRCW);
#ifdef DIAGS
                vfile("zsendfile: %d %s", blen, buf);
#endif
#ifdef LDP
                if (Zldpactive) {
                        for (p=buf, c=blen; --c >=0; ++p) {
                                crc = UPDC32(*p, crc);
#ifdef DIAGS
                                vfile("crc=%lX", crc);
#endif
                        }
                        for (p = zcpyrldp; *p; ++p)
                                crc = UPDC32(*p, crc);
                        Zcrcpreset = crc;
#ifdef DIAGS
                        vfile("Zcrcpreset=%lX", Zcrcpreset);
#endif
                }
#endif
again:
                c = zgethdr();
                switch (c) {
                case ZRINIT:
#ifdef DIAGS
                vfile("spurious ZRINIT");
#endif
                        while ((c = readline(50)) > 0)
                                if (c == ZPAD) {
                                        goto again;
                                }
                        continue;
                default:
                case ZCAN:
                case TIMEOUT:
                case ZABORT:
                case ZFIN:
                        return ERROR;
                case ZSKIP:
#ifdef DIAGS
                        vfile("1zskip");
#endif
                        zclose(ERROR); return c;
                case ZRPOS:
                        /*
                         * Suppress zcrcw request otherwise triggered by
                         * lastyunc==bytcnt
                         */
                        if (Rxpos && zseek(Rxpos))
                                return ERROR;
                        Lastsync = (bytcnt = Txpos = Rxpos) -1;
                        return zsendfdata();
                }
        }
}

/* Send the data in the file */
zsendfdata()
{
        register c, e, n;
        register newcnt;
        int junkcount;          /* Counts garbage chars received by TX */

        Lrxpos = 0;
        junkcount = 0;
        Beenhereb4 = FALSE;
        goto somemore;

waitack:
        junkcount = 0;
        c = getinsync(0);
gotack:
#ifdef DIAGS
        vfile("gotack: c=%d", c);
#endif
        switch (c) {
        default:
        case ZCAN:
#ifdef DIAGS
                vfile("zcan");
#endif
                zclose(ERROR);
                return ERROR;
        case ZSKIP:
#ifdef DIAGS
                vfile("2zskip");
#endif
                zclose(ERROR);
                return c;
        case ZACK:
        case ZRPOS:
                break;
        case ZRINIT:
#ifdef DIAGS
                vfile("spurious ZRINIT");
#endif
                zclose(ERROR);
                return OK;
        }
        while (checkline()) {
                switch (readline(1))
                {
                case XOFF:              /* Wait a while for an XON */
                case XOFF|0200:
                        readline(100); continue;
                case CAN:
                case ZPAD:
                        c = getinsync(1);
                        goto gotack;
                }
        }

somemore:
        newcnt = Rxbuflen;
        Txwcnt = 0;
        e = ZCRCE;


        do {
                if (e == ZCRCE) {
                        stohdr(Txpos);
                        zsbhdr(ZDATA, Txhdr);
                }
                n = zfilbuf();
                if (Eofseen)
                        e = ZCRCE;
                else if (junkcount > 3)
                        e = ZCRCW;
                else if (bytcnt == Lastsync)
                        e = ZCRCW;
                else if (Rxbuflen && (newcnt -= n) <= 0)
                        e = ZCRCW;
                else if (Txwindow && (Txwcnt += n) >= Txwspac) {
                        Txwcnt = 0;  e = ZCRCQ;
                }
                else
                        e = ZCRCG;
#ifdef UNIX
                if (Verbose>1)
                        fprintf(stderr, "%7ld ZMODEM%s\n", Txpos,
#ifndef XEDCRC32
                        Crc32t?" CRC-32":"CRC-16");
#else
                        "CRC-16");
#endif
#endif
                zsdata(secbuf, n, e);
                bytcnt = Txpos += n;
                if (e == ZCRCW)
                        goto waitack;
                while (checkline()) {
                        switch (readline(1))
                        {
                        case CAN:
                        case ZPAD:
                                c = getinsync(1);
                                if (c == ZACK)
                                        break;
#ifdef TCFLSH
                                ioctl(iofd, TCFLSH, 1);
#endif
                                /* zcrce - dinna wanna starta ping-pong game */
                                zsdata(secbuf, 0, ZCRCE);
                                goto gotack;
                        case XOFF:              /* Wait a while for an XON */
                        case XOFF|0200:
                                readline(100);
                        default:
                                ++junkcount;
                        }
                }
                if (Txwindow) {
                        while ((Txpos - Lrxpos) >= Txwindow) {
                                if (e != ZCRCQ)
                                        zsdata(secbuf, 0, e = ZCRCQ);
                                c = getinsync(1);
                                if (c != ZACK) {
#ifdef TCFLSH
                                        ioctl(iofd, TCFLSH, 1);
#endif
                                        zsdata(secbuf, 0, ZCRCE);
                                        goto gotack;
                                }
                        }
                }
        } while (!Eofseen);

        stohdr(Txpos);
        zshhdr(ZEOF, Txhdr);
        for (;;) {
                switch (getinsync(0)) {
                case ZACK:
                        continue;
                case ZRPOS:
                        goto somemore;
                case ZRINIT:
#ifdef DIAGS
                        vfile("ZRINIT on eof");
#endif

#ifdef LDP
#ifdef DIAGS
                        vfile("ZGOTFILEOK=%d", Rxhdr[ZF2]);
#endif
                        if ((!Zldpactive) || (Rxhdr[ZF2] & ZGOTFILEOK)) {  /* Last file received and closed */
                                zclose(OK);
                                return OK;
                        } else {
                                zclose(ERROR);
                                return ERROR;
                        }
#else
                        zclose(OK);
                        return OK;
#endif

                case ZSKIP:
#ifdef DIAGS
                        vfile("3zskip");
#endif
                        zclose(ERROR);
                        return c;
                default:
#ifdef DIAGS
                        vfile("DEFAULT on eof");
#endif
                        zclose(ERROR);
                        return ERROR;
                }
        }
}

/*
 * Respond to receiver's complaint, get back in sync with receiver
 */
getinsync(flag)
{
        register c;

        for (;;) {
                c = zgethdr();
                switch (c) {
                case ZCAN:
                case ZABORT:
                case ZFIN:
                case TIMEOUT:
                        return ERROR;
                case ZRPOS:
                        /* ************************************* */
                        /*  If sending to a buffered modem, you  */
                        /*   might send a break at this point to */
                        /*   dump the modem's buffer.            */
                        if (zseek(Rxpos))
                                return ERROR;
                        Eofseen = 0;
                        bytcnt = Lrxpos = Txpos = Rxpos;
                        if (Lastsync == Rxpos) {
                                if (++Beenhereb4 > 4)
                                        if (blklen > 32)
                                                blklen /= 2;
                        }
                        Lastsync = Rxpos;
                        return c;
                case ZACK:
                        Lrxpos = Rxpos;
                        if (flag || Txpos == Rxpos)
                                return ZACK;
                        continue;
                case ZRINIT:
#ifdef DIAGS
                        vfile("zrinit2");
#endif
                        return c;
                case ZSKIP:
#ifdef DIAGS
                        vfile("4zskip");
#endif
                        return c;
                case ERROR:
                default:
                        zshhdr(ZNAK, Txhdr);
                        continue;
                }
        }
}


/* Say "bibi" to the receiver, try to do it cleanly */
void
saybibi()
{
        for (;;) {
                stohdr(0L);             /* CAF Was zsbhdr - minor change */
                zshhdr(ZFIN, Txhdr);    /*  to make debugging easier */
                switch (zgethdr()) {
                case ZFIN:
                        sendline('O'); sendline('O');
                /*FALLTHRU*/
                case ZCAN:
                case TIMEOUT:
                        return;
                }
        }
}



/* End of isz.c */
