/*
 *   Z M O D E M . H     Manifest constants for ZMODEM
 *    application to application file transfer protocol
 *  Copyright 2002 Omen Technology INC All Rights Reserved
 */
#define ZPAD '*'        /* 052 Padding character begins frames */
#define ZDLE 030        /* Ctrl-X Zmodem escape - `ala BISYNC DLE */
#define ZDLEE (ZDLE^0100)       /* Escaped ZDLE as transmitted */
#define ZBIN 'A'        /* Binary frame indicator */
#define ZHEX 'B'        /* HEX frame indicator */
#define ZBIN32 'C'      /* Binary frame with 32 bit FCS */

/* Frame types (see array "frametypes" in izm.c) */
#define ZRQINIT 0       /* Request receive init */
#define ZRINIT  1       /* Receive init */
#define ZSINIT 2        /* Send init sequence (optional) */
#define ZACK 3          /* ACK to above */
#define ZFILE 4         /* File name from sender */
#define ZSKIP 5         /* To sender: skip this file */
#define ZNAK 6          /* Last packet was garbled */
#define ZABORT 7        /* Abort batch transfers */
#define ZFIN 8          /* Finish session */
#define ZRPOS 9         /* Resume data trans at this position */
#define ZDATA 10        /* Data packet(s) follow */
#define ZEOF 11         /* End of file */
#define ZFERR 12        /* Fatal Read or Write error Detected */
#define ZCRC 13         /* Request for file CRC and response */
#define ZCHALLENGE 14   /* Receiver's Challenge */
#define ZCOMPL 15       /* Request is complete */
#define ZCAN 16         /* Other end canned session with CAN*5 */
#define ZFREECNT 17     /* Request for free bytes on filesystem */
#define ZCOMMAND 18     /* Command from sending program */

/* ZDLE sequences */
#define ZCRCE 'h'       /* CRC next, frame ends, header packet follows */
#define ZCRCG 'i'       /* CRC next, frame continues nonstop */
#define ZCRCQ 'j'       /* CRC next, frame continues, ZACK expected */
#define ZCRCW 'k'       /* CRC next, ZACK expected, end of frame */
#define ZRUB0 'l'       /* Translate to rubout 0177 */
#define ZRUB1 'm'       /* Translate to rubout 0377 */

/* zdlread return values (internal) */
/* -1 is general error, -2 is timeout */
#define GOTOR 0400
#define GOTCRCE (ZCRCE|GOTOR)   /* ZDLE-ZCRCE received */
#define GOTCRCG (ZCRCG|GOTOR)   /* ZDLE-ZCRCG received */
#define GOTCRCQ (ZCRCQ|GOTOR)   /* ZDLE-ZCRCQ received */
#define GOTCRCW (ZCRCW|GOTOR)   /* ZDLE-ZCRCW received */
#define GOTCAN  (GOTOR|030)     /* CAN*5 seen */

/* Byte positions within header array */
#define ZF0     3       /* First flags byte */
#define ZF1     2
#define ZF2     1
#define ZF3     0
#define ZP0     0       /* Low order 8 bits of position */
#define ZP1     1
#define ZP2     2
#define ZP3     3       /* High order 8 bits of file position */

/* Bit Masks for ZRINIT flags byte ZF0 */
#define CANFDX  01      /* Rx can send and receive true FDX */
#define CANOVIO 02      /* Rx can receive data during disk I/O */
#define CANBRK  04      /* Rx can send a break signal */
#define CANCRY  010     /* Receiver can decrypt */
#define CANLZW  020     /* Receiver can uncompress */
#define CANFC32 040     /* Receiver can use 32 bit Frame Check */
#define ESCCTL 0100     /* Receiver expects ctl chars to be escaped */
#define ESC8   0200     /* Receiver expects 8th bit to be escaped */

/* Bit Masks for ZRINIT flags byte ZF1 */
#define CANLDP  128     /* Receiver understands LDP protection */

/* Bit Masks for ZRINIT flags byte ZF2 */
#define ZGOTFILEOK 1    /* Last file received and closed */

/* Parameters for ZSINIT frame */
#define ZATTNLEN 32     /* Max length of attention string */
/* Bit Masks for ZSINIT flags byte ZF0 */
#define TESCCTL 0100    /* Transmitter expects ctl chars to be escaped */
#define TESC8   0200    /* Transmitter expects 8th bit to be escaped */

/* Parameters for ZFILE frame */
/* Conversion options one of these in ZF0 */
#define ZCBIN   1       /* Binary transfer - inhibit conversion */
#define ZCNL    2       /* Convert NL to local end of line convention */
#define ZCRESUM 3       /* Resume interrupted file transfer */
/* Management include options, one of these ored in ZF1 */
#define ZMSKNOLOC       0200    /* Skip file if not present at rx */
/* Management options, one of these ored in ZF1 */
#define ZMMASK  037     /* Mask for the choices below */
#define ZMNEWL  1       /* Transfer if source newer or longer */
#define ZMCRC   2       /* Transfer if different file CRC or length */
#define ZMAPND  3       /* Append contents to existing file (if any) */
#define ZMCLOB  4       /* Replace existing file */
#define ZMNEW   5       /* Transfer if source newer */
        /* Number 5 is alive ... */
#define ZMDIFF  6       /* Transfer if dates or lengths different */
#define ZMPROT  7       /* Protect destination file */
/* Transport options, one of these in ZF2 */
#define ZTLZW   1       /* Lempel-Ziv compression */
#define ZTCRYPT 2       /* Encryption */
#define ZTRLE   3       /* Run Length encoding */
/* Extended options for ZF3, bit encoded */
#define ZSLDP   16      /* CRCs will be biased for LDP protection */
#define ZXSPARS 64      /* Encoding for sparse file operations */

/* Parameters for ZCOMMAND frame ZF0 (otherwise 0) */
#define ZCACK1  1       /* Acknowledge, then do command */

/* Globals used by ZMODEM functions */
extern Rxframeind;      /* ZBIN ZBIN32, or ZHEX type of frame received */
extern Rxtype;          /* Type of header received */
extern Rxcount;         /* Count of data bytes received */
extern long bytcnt;
extern Zrwindow;        /* RX window size (controls garbage count) */
extern Rxtimeout;       /* Tenths of seconds to wait for something */
extern char Rxhdr[4];   /* Received header */
extern char Txhdr[8];   /* Transmitted header */
extern long Rxpos;      /* Received file position */
extern long Txpos;      /* Transmitted file position */

#ifndef XEDCRC32
extern Txfcs32;         /* TURE means send binary frames with 32 bit FCS */
extern Crc32t;          /* Display flag indicating 32 bit CRC being sent */
#endif

#ifdef LDP
extern Zldpactive;      /* LDP protection active */
extern char Zaddr8[8];  /* Array for address CRC bias */
extern unsigned long Zcrcpreset;        /* CRC preset from filename header */
extern unsigned long Zcrcinit;  /* CRC init for zrdata zsdata */
extern char zcpyrldp[];
int Zfileclosedok;      /* last file was received and closed ok */
#endif

#ifndef XEDCRC32
void  zsda32();
#endif

/* End of ZMODEM.H */

/* isz.c defines */
#define OK 0
#define FALSE 0
#define TRUE 1
#define ERROR (-1)
/* Ward Christensen / CP/M parameters - Don't change these! */
#define CAN ('X'&037)
#define XOFF ('s'&037)
#define XON ('q'&037)
#define TIMEOUT (-2)
#define xsendline(c) sendline(c)

/* isz.c prototypes */
int SZ(char *fname, char *fsize);
int startz(void);
int wcs(char *name, char *size);
int wctxpn(char *name, char *size);
int getnak(void);
int zclose(int status);
int zseek(long n);
int zfilbuf(void);
int getzrxinit(void);
int zsendfile(char *buf, int blen);
int zsendfdata(void);
int getinsync(int flag);
void saybibi(void);

/* izm.c prototypes */
void zsbh32(char *hdr, int type);
void zsbhdr(int type, char *hdr);
void zshhdr(int type, char *hdr);
void zsda32(char *buf, int length, int frameend);
void zsdata(char *buf, int length, int frameend);
int zrdata(char *buf, int length);
int zrdat32(char *buf, int length);
int zgethdr(void);
int zrbhdr(void);
int zrbhdr32(void);
int zrhhdr(void);
void zputhex(int c);
void zsendline(int c);
int zgethex(void);
int zgeth1(void);
int zdlread(void);
int noxrd7(void);
void stohdr(long pos);
long rclhdr(char *hdr);
void canit(void);

/* tt8_zmod.c prototypes */
void sendline(char c);
void mputs(char *s);
void purgeline(void);
int readline(int timeout);
int checkline(void);
void WrUsbDataByte(unsigned char arg_val);
unsigned char RdUsbDataByte(void);
unsigned char RdUsbSleepPin(void);
unsigned char RdUsbRxfPin(void);
unsigned char RdUsbTxePin(void);

